#include "virge.h"

int sophie_scanfile(char *scanpath)
{
	int sophie_sock;
	int sophie_bread;
	struct sockaddr_un sophie_server;
	char sophie_buffer[V_BUFSIZE];
	char sophie_sockname[V_FILESIZE];
	
	/* Create a socket */
	if ( (sophie_sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
	{
		virge_syslog("FAILURE: Failed to create a socket in scan_sophie()");
		virge_debug(2, "[ scan_sophie() ] failed to create socket");
		return(99);
	}
	
	/* Check if socket name exists */
	if (strlen(V_SCAN_SOPHIE_SOCKET) < 1)
	{
		virge_syslog("FAILURE: V_SCAN_SOPHIE_SOCKET name not set");
		virge_debug(2, "[ scan sophie() ] FAILURE: V_SCAN_SOPHIE_SOCKET name not set");
		return(99);
	}
	
	/* Copy the name into sophie_sockname */
	strncpy(sophie_sockname, V_SCAN_SOPHIE_SOCKET, sizeof(sophie_sockname)-1);
	
	/* Set the socket properties */
	sophie_server.sun_family = AF_UNIX;
	strncpy(sophie_server.sun_path, V_SCAN_SOPHIE_SOCKET, sizeof(sophie_server.sun_path)-1);
	virge_debug(3, "[ scan sophie() ] V_SCAN_SOPHIE_SOCKET (%s) properties set", V_SCAN_SOPHIE_SOCKET);

	/* Connect */
	if (connect(sophie_sock, (struct sockaddr *) &sophie_server, sizeof(struct sockaddr_un)) < 0)
	{
		virge_syslog("FAILURE: connect() to socket failed in scan_sophie()");
		virge_debug(2, "[ scan sophie() ] FAILURE: V_SCAN_SOPHIE_SOCKET name not set");
		return(99);
	}

	/* Send a filename path to scan */
	if (write(sophie_sock, scanpath, strlen(scanpath)) < 0)
	{
		perror("write");
		virge_syslog("FAILURE: write() to socket failed in scan_sophie() [tried to write: (%s)]", scanpath);
		virge_debug(2, "[ scan sophie() ] FAILURE: write() to socket failed [tried to write: (%s)]", scanpath);
		return(99);
	}

	/* Read the response */
	memset(sophie_buffer, 0, sizeof(sophie_buffer));
	if ((sophie_bread = read(sophie_sock, sophie_buffer, sizeof(sophie_buffer))) > 0)
	{
		if (strchr(sophie_buffer, '\n'))
			*strchr(sophie_buffer, '\n') = '\0';
		
		/* Set the VIRUS_NAME */
		if (index(sophie_buffer, ':') != NULL)
			strncpy(VIRUS_NAME, index(sophie_buffer, ':')+1, sizeof(VIRUS_NAME)-1);

		return(atoi(sophie_buffer));
		close(sophie_sock);
	}
	else
	{
		virge_syslog("FAILURE: read() from socket failed in scan_sophie()");
		virge_debug(2, "[ scan sophie() ] FAILURE: read() from socket failed");
		close(sophie_sock);
		return(99);
	}

	/* This couldn't be reached, I presume */
	close(sophie_sock);

}

/* Send filepath(s) from scratch directory to Sophie daemon for scanning */
int sophie_scandir(char *scan_dir)
{
	int sophie_resp;
	char scan_prepath[V_FILESIZE];

	struct dirent *dir_contents;
	DIR *dir;
	
	/* Open the directory */
	if (!(dir = opendir(scan_dir)))
	{
		virge_syslog("FAILURE: opendir() failed for '%s'", scan_dir);
		return(0);
	}

	/* Browse through files, and scan them */
	for (dir_contents = readdir(dir) ; dir_contents != NULL ; dir_contents = readdir(dir))
	{
		if ( (strcmp(dir_contents->d_name, ".")) && (strcmp(dir_contents->d_name, "..")) )
		{
			/* Add trailing slash if it's missing (for whatever reason) */
			if (!(strcmp(scan_dir+(strlen(scan_dir)-1), "/")))
				snprintf(scan_prepath, sizeof(scan_prepath)-1, "%s%s", scan_dir, dir_contents->d_name);
			else
				snprintf(scan_prepath, sizeof(scan_prepath)-1, "%s/%s", scan_dir, dir_contents->d_name);

			virge_debug(3, "[ sophie_scandir() ] Scanning file : [%s]", scan_prepath);
			sophie_resp = sophie_scanfile(scan_prepath);
			virge_debug(3, "[ sophie_scandir() ] Scan response = [%d])", sophie_resp);

			if (sophie_resp == 1)
				return(1);
		}
	}
	
	closedir(dir);

	return(0);
}
