#include "virge.h"
#include "virge_main.h"

static char tempfile[V_FILESIZE];

/*
	TODO:
	- Clean up debugging/logging in Virge - it's *major* mess right now
	- Make 'notice' messages (sent to admin/whoever) be read from file (as a template)?
	- Sending notification mail over SMTP (would you want that, since virge is already running on mail server? :)
	- Can 'fixed' arrays (foo[CONFIG_MAX][CONFIG_LEN]) be created in 'dynamic' way? Will it be benefitial?
	- ... fix heaps of bugs... :)
*/

void virge_timeout(int sig);
void sig_exit(int sig);

int main(int argc, char *argv[])
{
	/* We'll fix the declarations later - hopefully :) */
	
	char mailbuf[V_BUFSIZE];
	char rtempfile[V_FILESIZE];
	char arg_a[V_ARGSIZE];
	char storageDir[V_FILESIZE];
	static char scratchdir[V_FILESIZE];
	char random[V_ARGSIZE];

	static int ISOLATE = 0;

	int fd;
	int c;
	int loadavg = 0;
	int avp_scanret = 0;
	int sophie_scanret = 0;
	int allowUserCurrent = 0;

	unsigned long bytes_read;
	unsigned long bytes_written;

	int removedPos;
	char removedName[CONFIG_LEN];

	struct passwd *userInfo;

	FILE *randfp;
	int randlen;
	int randc;

	time_t msgid_current;
	struct tm *msgid_now;
	char msgid_time[32];

	struct rfc2045 *rfcbody = rfc2045_alloc();
	void (*do_extract)(struct rfc2045 *, const char *, int, char **) = 0;

	/* Signal handling stuff */
	int sig_ret;
	struct sigaction sa_exit;

	/* Get the program name (without leading path) as specified in sendmail.cf - will be used for syslog-ing :) */
	/* No, I don't like to use char *progname; progname = argv[0]; */
	strncpy(V_PROGNAME, argv[0], sizeof(V_PROGNAME)-1);

	MAIL_FROM[0] = '\0';
	RCPT_TO[0] = '\0';
	arg_a[0] = '\0';

	/* Check/get the arguments/options */
	while ((c = getopt(argc, argv, "c:Cf:a:d:s:hD:L:R")) != EOF)
	{
		switch (c)
		{
			/* Config file location */
			case 'c':
			if (strlen(optarg) < 1)
			{
				fprintf(stderr, "ERROR: '-c' used, but no configuration file specified - aborting!\n");
				exit(1);
			}
			else if (!file_exists(optarg))
			{
				fprintf(stderr, "ERROR: Configuration file '%s' does not exist - aborting!\n", optarg);
			}
			else
			{
				strncpy(V_CONFIG, optarg, sizeof(V_CONFIG)-1);
			}
			break;
			
			/* Config check requested? */
			case 'C':
			CHECK_CONFIG_VERBOSE = 1;
			break;

			/* Sender - as given by sendmail (-f <email>) */
			case 'f':
			strncpy(MAIL_FROM, optarg, sizeof(MAIL_FROM)-1);
			virge_debug(1, "[ main() ] MAIL_FROM set to: '%s'", MAIL_FROM);
			break;

			/* -a option for procmail */
			case 'a':
			strncpy(arg_a, optarg, sizeof(arg_a)-1);
			virge_debug(1, "[ main() ] arg_a set to: '%s'", arg_a);
			break;
			
			/* this will contain 1st 'from' header - used for local delivery */
			/* This will be set if the -f was not specified by sendmail */
			case 's':
			if (strlen(MAIL_FROM) < 1)
			{
				strncpy(MAIL_FROM, optarg, sizeof(MAIL_FROM)-1);
				virge_debug(1, "[ main() ] MAIL_FROM set to: '%s' using content from '-s', because Sendmail didn't supply the sender using '-f'", RCPT_TO);
			}
			break;

			/* Recipient */
			case 'd':
			strncpy(RCPT_TO, optarg, sizeof(RCPT_TO)-1);
			virge_debug(1, "[ main() ] RCPT_TO set to: '%s'", RCPT_TO);
			break;
			
			/* DEBUG = ? */
			case 'D':
			if (isdigit((int) *optarg))
				DEBUG = atoi(optarg);
			else
				DEBUG = 1;
			CMDLINE_DEBUG = 1;
			break;

			/* DEBUG_LEVEL = ? */
			case 'L':
			if (isdigit((int) *optarg))
				DEBUG_LEVEL = atoi(optarg);
			else
				DEBUG_LEVEL = 1;
			CMDLINE_DEBUG_LEVEL = 1;
			break;
			
			/* Check regexp in extension file */
			case 'R':
			CHECK_REGEX = 1;
			break;

			/* Help */
			case 'h':
			usage();
			break;
		}
	}

	/* Check that VIRGE_DIR_MODE is not empty */
	/* If it is - fill it with 00700 */
	if (VIRGE_DIR_MODE[0] == '\0')
		strncpy(VIRGE_DIR_MODE, "00700", 5);

	/* Set signal handlers */
/*	bzero((char *)(&sa_exit), sizeof(sa_exit)); */
	sigemptyset(&sa_exit.sa_mask);
	sa_exit.sa_handler = sig_exit;
	sa_exit.sa_flags = SA_RESETHAND;

	if ((sig_ret = sigaction(SIGINT, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGINT");

	if ((sig_ret = sigaction(SIGTERM, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGTERM");

	if ((sig_ret = sigaction(SIGSEGV, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGSEGV");

	if ((sig_ret = sigaction(SIGILL, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGILL");

	if ((sig_ret = sigaction(SIGBUS, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGBUS");

	if ((sig_ret = sigaction(SIGTRAP, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGTRAP");

	if ((sig_ret = sigaction(SIGQUIT, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGQUIT");

	/* I *think* these ones should be ignored  - they shouldn't really show up */
	sa_exit.sa_handler = SIG_IGN;

	if ((sig_ret = sigaction(SIGCHLD, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGCHLD");

	if ((sig_ret = sigaction(SIGHUP, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGHUP");

	/* If virge times out (hangs) for a while, it needs to die and alert someone about the issue */
	sa_exit.sa_handler = virge_timeout;

	if ((sig_ret = sigaction(SIGALRM, &sa_exit, NULL)) < 0)
		virge_syslog(0, "Error setting handler for SIGALRM");

	/* Let's check that V_CONFIG exists - if not, try to use "/etc/virge.conf" by default */
	if (strlen(V_CONFIG) < 1)
		strncpy(V_CONFIG, "/etc/virge.conf", sizeof(V_CONFIG)-1);

	/* load configuration from file */
	load_virge_config();

	/* Set the timeout */
	alarm(VIRGE_TIMEOUT);

	/* Check current Virge configuration (as loaded from a config file) */
	check_current_config();

	/* Do we need to test extensions for regular expression validity? */
	if (CHECK_REGEX)
	{
		check_regex();
		exit(0);
	}

	/* Get the uid/gid for RUN_AS user - for dropping privileges */
	if (!(userInfo = getpwnam(RUNAS_USER)))
	{
		virge_debug(2, "[ main() ] failed to obtain UID/GID for user '%s'", RUNAS_USER);
		virge_syslog("FAILURE: Failed to obtain UID/GID for user '%s' - RUNNING AS ROOT", RUNAS_USER);
	}
	else
	{
		virge_debug(1, "[ main() ] running as user '%s'", RUNAS_USER);
		/*
		  Drop the privileges (hopefully ;)
		  We don't need to run as root...
		*/
		if ((setgid(userInfo->pw_gid)) == -1)
		{
			virge_debug(2, "[ main() ] failed to setgid() to [%d]", userInfo->pw_gid);
			virge_syslog("FAILURE: Failed to setgid to '%d' - RUNNING AS ROOT", userInfo->pw_gid);
		}
		else
		{
			virge_debug(3, "[ main() ] managed to setgid() to [%d]", userInfo->pw_gid);
		}

		if ((setuid(userInfo->pw_uid)) == -1)
		{
			virge_debug(2, "[ main() ] failed to setuid() to [%d]", userInfo->pw_uid);
			virge_syslog("FAILURE: Failed to setuid to '%d - RUNNING AS ROOT'", userInfo->pw_uid);
		}
		else
		{
			virge_debug(3, "[ main() ] managed to setuid() to [%d]", userInfo->pw_uid);
		}

	}

	/* if '-a' was empty - put quotes there, so that -a gets passed to procmail properly later (hope this is the right way :) */
	if (strncmp(arg_a, "", 1) == 0)
	{
		virge_debug(1, "[ main() ] '-a' was empty - setting to '' (2 single quotes :)");
		strcpy(arg_a, "''");
	}

	/* Make some more 'serious' checks for recipient/sender - FIXME */
	/* Do I care about it at all? */
	if (RCPT_TO[0] == '\0')
	{
		/* We will isolate this mail */
		ISOLATE = 1;

		/* Put "UNKNOWN" as the recipient */
		strcpy(RCPT_TO, "UNKNOWN");
		virge_debug(2, "[ main() ] No recipient specified (ouch!) - setting it to 'UNKNOWN'");
	}

	/* Create a 'message ID' - just stupid time() string + pid, which we'll use in logs, etc. */
	msgid_current = time(NULL);
	msgid_now = localtime(&msgid_current);

	memset(msgid_time, 0, sizeof(msgid_time));
	snprintf(msgid_time, sizeof(msgid_time)-1, "%02d:%02d:%02d", msgid_now->tm_hour, msgid_now->tm_min, msgid_now->tm_sec);

	snprintf(messageID, sizeof(messageID)-1, "%s.%05d", msgid_time, getpid());
	virge_debug(1, "[ main() ] Message ID created: [%s]", messageID);

	/* Create a name for temporary file */
	snprintf(tempfile, sizeof(tempfile)-1, "%s/%s", V_TEMPDIR, "virge-XXXXXX");
	virge_debug(3, "[ main() ] tempfile name created: '%s'", tempfile);

	/* Open temporary file */
	if ((fd = mkstemp(tempfile)) == -1)
	{
		virge_debug(2, "[ main() ] failed to mkstemp(\"%s\")", tempfile);
		virge_syslog("FAILURE: Failed to create temporary file with mkstemp() in main()");
	}

	/* Read the input, and place it into the file */
	/* Also, rfc2045 parse it on the fly */
	while( (bytes_read = read(0, mailbuf, sizeof(mailbuf))) > 0)
	{
		virge_debug(3, "[ main() ] read [%lu] bytes from stdin into mailbuf", bytes_read);

		V_MAIL_SIZE += bytes_read;

		if ((bytes_written = write(fd, mailbuf, bytes_read)) == -1)
		{
			virge_debug(2, "[ main() ] failed to write() [%lu] bytes into fd", bytes_read);
			virge_syslog("FAILURE: Failed to write() [%lu] bytes into fd", bytes_read);
		}
		
		rfc2045_parse(rfcbody, mailbuf, sizeof(mailbuf));
	}

	/* Replace stdin */
	/* Needed for fseek() section - look for it in the rfcfuncs.c source... */
	if (fd)
	{
		close(0);
		dup(fd);
	}

	/* Close the temporary file */
	close(fd);
	virge_debug(3, "[ main() ] temporary file (%s) closed", tempfile);

	/* Load list of users for which mail should be delivered w/o checking */
	allowUserCount = load_allowusers();

	/* Check the 'direct' delivery (if mail needs to be checked at all - ALLOW_USERS */
	for (allowUserCurrent = 0; allowUserCurrent < allowUserCount; allowUserCurrent++)
	{
		if (!strncmp(ALLOW_USERS[allowUserCurrent], RCPT_TO, strlen(RCPT_TO)))
		{
			virge_debug(3, "[ main() ] delivering mail directly to '%s'", RCPT_TO);
			
			/* Delivery because mail doesn't need to be checked */
			virge_procmail(tempfile, arg_a, 1);
			
			rfc2045_free(rfcbody);
			unlink(tempfile);
			exit(0);
		}
	}

	/*
	   If recipient is missing, we will isolate the mail now - no need to decode attachments,
	   admin will (should :) probably check it anyway
	*/
	if (ISOLATE == 1)
	{
		/* Isolate the mailfile */
		virge_debug(2, "[ main() ] isolating mail because no recipients were specified");
		isolate(2, "No recipients have been specified.", tempfile, mkdir_isolate(storageDir), 1, "Mail has been isolated because no recipients have been specified.");
		rfc2045_free(rfcbody);
		unlink(tempfile);
		exit(0);
	}

	/* Check the load on the machine - if load exceeds the 'deliver_loadavg' param in config, deliver immediately */
	loadavg = check_loadavg();

	/* Main part... */
	
	/* Deliver immediately if load is high */
	if (!(loadavg < V_DELIVER_LOADAVG))
	{
		virge_debug(1, "[ main() ] Direct delivery (from: '%s', to: '%s') because of high load on the machine", MAIL_FROM, RCPT_TO);
		event_log(LOG_VIRGE, "Direct delivery [load too high] for mail from=<%s>, to=<%s>, size=<%lu bytes>", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);

		/* Delivery because load is high and we don't want to waste more resources */
		virge_procmail(tempfile, arg_a, 1);

		rfc2045_free(rfcbody);
		unlink(tempfile);
		exit(0);
	}

	/* Ok, load is less than V_DELIVER_LOADAVG - go on */

	/*
	  Get the headers of the mail (will need it later for sender notification)
	  
	  Damn. Why people want the notification feature, argh? ;)
	*/
	get_headers(tempfile);

	/* Decode the message... */
	rfc2045_decode(rfcbody, &do_print_structure, 0);

	/* Load the 'list' of extensions/filenames to check */
	extensionCount = load_extensions();
			
	/* If there is more than 1 section - unpack & scan the mail */
	/* If there is only 1 section - deliver it (no attachments - hopefully :), and avoid abusing resources :) */
	if (sections > 1)
	{
		/* 'Craft' the dir name where to extract file(s) */
		if ((randfp = fopen("/dev/urandom", "r")) != NULL)
		{
			int lc = 0;
			memset(random, 0, sizeof(random));
			for (randlen = 0; randlen <= 10; randlen++)
			{
				randc = fgetc(randfp);
				sprintf(&random[lc], "%.2x", randc);
				lc = lc + 2;
			}
			fclose(randfp);
			snprintf(scratchdir, sizeof(scratchdir)-1, "%s/%s/", V_TEMPDIR, random);
		}
		else
		{
			/* This will, most likely, apply to Solaris/HP-UX - and possibly some older Linux distros w/o /dev/urandom */
			/* Just don't complain... :) */
			char *temp_dir_name = 0;
			pid_t temp_pid;
			time_t temp_now;

			temp_dir_name = tempnam(V_TEMPDIR, "virge");
			temp_now = time(NULL);
			temp_pid = getpid();
			
			snprintf(scratchdir, sizeof(scratchdir)-1, "%s.%d%lu/", temp_dir_name, temp_pid, temp_now);
		}

		virge_debug(3, "[ main() ] scratch dir *name* created: '%s'", scratchdir);

		if (!virge_mkdir(scratchdir))
		{
			virge_debug(2, "[ main() ] fatal error - scratch dir not created - delivering directly");

			/* Delivery because scratchdir couldn't be created - I'd rather have mail delivered than risk to lose it (yes, even if it's virus ;) */
			virge_procmail(tempfile, arg_a, 1);

			rfc2045_free(rfcbody);
			unlink(tempfile);
			exit(0);
		}
			
		virge_debug(1, "[ main() ] scratch dir '%s' created", scratchdir);

		/* Set umask  */
		umask(0077);
		virge_debug(3, "[ main() ] umask() set");

		/* Ok - now extract files there... */
		do_extract = extract_file;
		extract_section(rfcbody, NULL, scratchdir, 0, NULL, do_extract);
		virge_debug(1, "[ main() ] attachments extracted into '%s'", scratchdir);

		/* Clear the virusname */
		memset(VIRUS_NAME, 0, sizeof(VIRUS_NAME));
		
		/* Now, we will scan for viruses. Check the V_SCAN_AVP setting first */
		if (V_SCAN_AVP)
		{
			avp_scanret = scan_avp(scratchdir);

			if (avp_scanret == 4)
			{
				if (VIRUS_NAME[0] != '\0')
					virge_debug(1, "[ main() ] AVP scan result: [%d] - virus name [%s]", avp_scanret, VIRUS_NAME);
				else
					virge_debug(1, "[ main() ] AVP scan result: [%d] - virusname not set (don't know why - yet)", avp_scanret);
			}
			else
			{
				virge_debug(1, "[ main() ] AVP scan result: [%d]", avp_scanret);
			}
		}
		else
		{
			virge_debug(1, "[ main() ] AVP scan has not been preformed (V_SCAN_AVP = 0)");
		}
			
		/* We will scan with Sophie only if AVP did not find virus already (so that we don't 'waste' resources) */
		if ( (avp_scanret != 4) && (V_SCAN_SOPHIE) )
		{
				sophie_scanret = sophie_scandir(scratchdir);
				virge_debug(1, "[ main() ] Sophie scan result: [%d]", sophie_scanret);
		}

		/* AVPDaemon returns 4 if virus is found. 0 is everything is clean */
		/* Sophie will return 1 */
		if ( (avp_scanret == 4) || (sophie_scanret == 1) )
		{
			if (VIRUS_NAME[0] != '\0')
			{
				virge_debug(1, "[ main() ] Virus [%s] found in mail from '%s' to '%s'", VIRUS_NAME, MAIL_FROM, RCPT_TO);
				event_log(LOG_ACTION, "Mail isolated (virus: %s) - from=<%s>, to=<%s>, size=<%lu bytes>", VIRUS_NAME, MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
				virge_syslog("WARNING: Virus [%s] has been found in mail from '%s' to '%s'", VIRUS_NAME, MAIL_FROM, RCPT_TO);

				/* Log this also virus logfile */
				event_log(LOG_VIRUS, "FROM: [%s] TO: [%s] SIZE: [%lu] VIRUS: [%s]", MAIL_FROM, RCPT_TO, V_MAIL_SIZE, VIRUS_NAME);
			}
			else
			{
				virge_debug(1, "[ main() ] Virus found in mail from '%s' to '%s'", MAIL_FROM, RCPT_TO);
				event_log(LOG_ACTION, "Mail isolated (virus) - from=<%s>, to=<%s>, size=<%lu bytes>", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
				virge_syslog("WARNING: Virus has been found in mail from '%s' to '%s'", MAIL_FROM, RCPT_TO);
			}

			isolate(0, "Virus has been found in the mail.", tempfile, mkdir_isolate(storageDir), 1, "Mail has been isolated because virus has been found.");
		}
		else if ( (avp_scanret != 0) && (avp_scanret != -1) )
		{
			/* Something else went wrong? */
			virge_debug(1, "[ main() ] Unexpected response from AVP: [%d]", avp_scanret);
			event_log(LOG_ACTION, "Mail isolated (unknown AVP response: [%d]) - from=<%s>, to=<%s>, size=<%lu bytes>", avp_scanret, MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
			virge_syslog("NOTE: Mail has been isolated - unknown response from AVP received [%d]", avp_scanret);
			isolate(0, "Unknown response from AVP received", tempfile, mkdir_isolate(storageDir), 1, "Unknown response from AVP has been received. Mail isolated.");
		}

		/* If extensionCount is 0 = don't bother to decode/rewrite (everything is allowed) */
		else if (extensionCount == 0)
		{
			/* Delivery because no 'extension' checking is required */
			virge_procmail(tempfile, arg_a, 1);
		}
		else
		{
			/* Check attachments, etc... */
			rfc2045_decode(rfcbody, &check_attachments, 0);

			if (REWRITE == 1)
			{
				/* Create temporary file (handle) for rewritten message */
				snprintf(rtempfile, sizeof(rtempfile)-1, "%s/%s", V_TEMPDIR, "rvirge-XXXXXX");

				/* Store the results into temporary file... */
				if ((rfd = mkstemp(rtempfile)) == -1)
				{
					virge_debug(2, "[ main() ] mkstemp(\"%s\") failed", rtempfile);
					virge_syslog("FAILURE: Failed to create temporary file with mkstemp() in rewrite_attachments()");
				}

				virge_debug(1, "[ main() ] temporary filename for rewritten message created: '%s'", rtempfile);
				rfc2045_decode(rfcbody, &rewrite_attachments, 0);

				/* Now, we have modified the message, we need to add 'dislaimer' at the end - it also helps us with the 'ending' boundary */
				write(rfd, rewriteHeader, strlen(rewriteHeader));

				/* Not very pretty, but this is just cosmetics anyway - fix it if you don't like it! :) */
				for (removedPos = 0; removedPos < removedCount; removedPos++)
				{
					/* Write the name of the stripped file */
					memset(removedName, 0, sizeof(removedName));
					snprintf(removedName, sizeof(removedName)-1, "| \"%s\"\n", REMOVED[removedPos]);
					write(rfd, removedName, strlen(removedName));
					virge_debug(3, "[ main() ] added '%s' filename to the 'removed' footer in the message", REMOVED[removedPos]);
				}

				/* More cosmetics... */
				write(rfd, "|\n", 2);
				write(rfd, "+------------------------------------------------------------ --- -- -\n", 71);


				/* And here comes the end boundary */
				write(rfd, boundary, strlen(boundary));
				write(rfd, "--\n", 3);

				/* Close the temporary file, with new mail contents */
				close(rfd);
				virge_debug(3, "[ main() ] rtempfile (%s) closed", rtempfile);
				
				/* Isolate the original mail (w/ attachment(s)) */
				virge_debug(1, "[ main() ] isolating mail (ID: '%s') because dodgy attachments found", messageID);
				isolate(1, "Dodgy attachments found in the mail", tempfile, mkdir_isolate(storageDir), 1, "Mail has been isolated because potentially dangerous attachment(s) have been found.");

				/* Delivery of the new (rewritten) mail */
				virge_procmail(rtempfile, arg_a, 0);

				/* Remove the temporary file */
				unlink(rtempfile);
				virge_debug(1, "[ main() ] rtempfile (%s) removed", rtempfile);
			}
			else
			{
				/* Delivery of checked mail - no rewriting required, no viruses found */
				virge_procmail(tempfile, arg_a, 0);
			}
		}

	}
	else
	{
		/* Normal mail - no attachments - deliver immediately */
		virge_procmail(tempfile, arg_a, 1);
	}

	/* Free some resources :) */
	rfc2045_free(rfcbody);
	virge_debug(1, "[ main() ] rfcbody freed");

	/* Remove the tempfile */
	unlink(tempfile);
	virge_debug(1, "[ main() ] tempfile (%s) removed", tempfile);

	/* Maybe to move this up? Will need it 2 times then... */
	if (sections > 1)
	{
		/* Remove the scratch directory */
		remove_tempdir(scratchdir);
		virge_debug(1, "[ main() ] scratchdir (%s) removed", scratchdir);
	}

	/* We didn't crash yet!?!? :) */
	exit(0);
}

void virge_timeout(int sig)
{
	syslog(LOG_ERR, "[VIRGE TIMEOUT] ==> PID [%d], MESSAGE ID [%s], MAIL_FROM [%s], RCPT_TO [%s], TEMPFILE [%s]", getpid(), messageID, MAIL_FROM, RCPT_TO, tempfile);
	exit(-1);
}

void sig_exit(int sig)
{
	syslog(LOG_ERR, "OUCH! We just caught signal '%d'. Mail from [%s] to [%s] IS NOT DELIVERED!!! (probably is still hanging somewhere in temporary directory [%s]).", sig, MAIL_FROM, RCPT_TO, V_TEMPDIR);
	exit(-1);
}
