#include "virge.h"

/*
  Checks filename (attachment) against the list of attachments 'denied' in configuration file (V_EXT)
*/

/* ### THIS IS NOW OBSOLETE - WILL PROBABLY BE REMOVED IN NEXT VERSION ### */
int check_extension(char *checkExtension)
{
	int extensionCurrent = 0;
	int cmpPosition;

	for (extensionCurrent = 0; extensionCurrent < extensionCount; extensionCurrent++)
	{
		virge_debug(4, "[ check_extension() ] comparing '%s' => '%s'", EXTENSIONS[extensionCurrent], checkExtension);
		if ((cmpPosition = strlen(checkExtension) - strlen(EXTENSIONS[extensionCurrent])) >= 0)
		{
			if (!strncasecmp(&checkExtension[cmpPosition], EXTENSIONS[extensionCurrent], strlen(EXTENSIONS[extensionCurrent])))
			{
				virge_debug(1, "[ check_extension() ] match '%s' => '%s'", EXTENSIONS[extensionCurrent], checkExtension);
				return(1);
			}
		}
	}
	
	virge_debug(1, "[ check_extension() ] extension checking finished - no matches");
	return(0);
}

/*
  Checks filename (attachment) against the list of attachements 'denied' in configuration file (V_EXT)
  This assumed extensions are regular expressions, and does regexp matching
*/
int check_extension_regex(char *regexCheckExtension)
{
	int regexExtensionCurrent = 0;
/*	int regexCmpPosition; */
	int regexErr;
	int regexExecStatus;

	regex_t preg;
	regmatch_t pmatch;

	for (regexExtensionCurrent = 0; regexExtensionCurrent < extensionCount; regexExtensionCurrent++)
	{
		virge_debug(4, "[ check_extension_regex() ] comparing '%s' => '%s'", EXTENSIONS[regexExtensionCurrent], regexCheckExtension);

		/* Compile regular expression - it is case insensitive */
		if ((regexErr = regcomp(&preg, EXTENSIONS[regexExtensionCurrent], REG_ICASE | REG_EXTENDED)))
		{
			/*
			  If 0 is not returned - some error occured
			  Expressions are dodgy - let's make sure some normal error messages are returned to the user
			  Hope it helps ;)
			*/
			switch (regexErr)
			{
				case REG_BADRPT:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_BADRPT);
				break;
				
				case REG_BADBR:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_BADBR);
				break;
				
				case REG_EBRACE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_EBRACE);
				break;
				
				case REG_EBRACK:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_EBRACK);
				break;
				
				case REG_ERANGE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ERANGE);
				break;
				
				case REG_ECTYPE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ECTYPE);
				break;
				
				case REG_ECOLLATE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ECOLLATE);
				break;
				
				case REG_EPAREN:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_EPAREN);
				break;
				
				case REG_ESUBREG:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ESUBREG);
				break;
				
/* Not on BSD */
#ifdef LINUX
				case REG_EEND:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_EEND);
				break;
#endif
				case REG_EESCAPE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_EESCAPE);
				break;
				
				case REG_BADPAT:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_BADPAT);
				break;
				
/* Not on BSD */
#ifdef LINUX
				case REG_ESIZE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ESIZE);
				break;
#endif
				case REG_ESPACE:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - error message \"%s\"", EXTENSIONS[regexExtensionCurrent], ERR_REG_ESPACE);
				break;
				
				default:
				virge_syslog("REGEX ERROR (regcomp): Extension (%s) invalid - unknown error", EXTENSIONS[regexExtensionCurrent]);
				break;
			}
		}
		else
		{
			/* 0 is returned for a match */
			if ((regexExecStatus = regexec(&preg, regexCheckExtension, 1, &pmatch, 0)) == 0)
			{
				virge_debug(1, "[ check_extension_regex() ] match '%s' => '%s'", EXTENSIONS[regexExtensionCurrent], regexCheckExtension);
	
				/* We have to free it before we return */
				regfree(&preg);
				return(1);
			}
		}
	
		/* Free preg */
		regfree(&preg);
	}
	
	virge_debug(1, "[ check_extension_regex() ] extension checking finished - no matches");
	return(0);
}

/*
  Checks attachments in MIME message - rfc2045_decode() 'loops' through all of them, and invokes this function for each
*/
void check_attachments(struct rfc2045 *p, struct rfc2045id *id, void *ptr)
{
	const char *content_type, *transfer_encoding, *charset;
	off_t start, end, body;
	const char *disposition;
	const char *disposition_name;
	const char *disposition_filename;
	const char *content_name;
	off_t nlines, nbodylines;

	rfc2045_mimeinfo(p, &content_type, &transfer_encoding, &charset);
	rfc2045_mimepos(p, &start, &end, &body, &nlines, &nbodylines);
	rfc2045_dispositioninfo(p, &disposition, &disposition_name, &disposition_filename);
	content_name=rfc2045_contentname(p);

/*	printf("START: [%lu] - END: [%lu] - BODY: [%lu] - LINES: [%lu] - TYPE: [%s]\n", (unsigned long)start, (unsigned long)end, (unsigned long)body, (unsigned long)nlines, content_type); */

	/* I hope this is safe way of passing things - if very big filename is supplied, seems like rfc2045 ignores it (didn't check for sure yet :) */
	if ( (disposition_filename && *disposition_filename) && (REWRITE != 1) )
	{
		virge_debug(3, "[ check_attachments() ] check disposition_filename: '%s'", disposition_filename);
		REWRITE = check_extension_regex((char *)disposition_filename);
	}
	else if ( (disposition_name && *disposition_name) && (REWRITE != 1) )
	{
		virge_debug(3, "[ check_attachments() ] check disposition_name: '%s'", disposition_name);
		REWRITE = check_extension_regex((char *)disposition_name);
	}
	else if ( (content_name && *content_name) && (REWRITE != 1) )
	{
		virge_debug(3, "[ check_attachments() ] check content_name: '%s'", content_name);
		REWRITE = check_extension_regex((char *)content_name);
	}

	virge_debug(1, "[ check_attachments() ] status of check_attachments (REWRITE) for content-type: [%s] is [%d]", content_type, REWRITE);
}

/*
  Bastard - took me a while, but it seems to be able to rewrite just about any MIME message I could feed.
  
  Now, the problem is that other people will start telling me that it won't rewrite *their* messages, argh... :)
*/
void rewrite_attachments(struct rfc2045 *p, struct rfc2045id *id, void *ptr)
{
	const char *content_type, *transfer_encoding, *charset;
	off_t start, end, body;
	const char *disposition;
	const char *disposition_name;
	const char *disposition_filename;
	const char *content_name;
	off_t nlines, nbodylines;

	char *readData;
	unsigned long cur;
	unsigned long sizeRead;
	unsigned long sizeWritten;
	
	int skipFile = 0;
	char boundaryString[V_BUFSIZE];

	rfc2045_mimeinfo(p, &content_type, &transfer_encoding, &charset);
	rfc2045_mimepos(p, &start, &end, &body, &nlines, &nbodylines);
	rfc2045_dispositioninfo(p, &disposition, &disposition_name, &disposition_filename);
	content_name=rfc2045_contentname(p);

	virge_debug(4, "[ rewrite_attachments() ] START: [%lu] - END: [%lu] - LINES: [%lu] - TYPE: [%s]", (unsigned long)start, (unsigned long)end, (unsigned long)nlines, content_type);

	/* We will skip completely the section '0' - it contains info about whole message */
	if (attachCounter != 0)
	{
		/* If attachCounter is 1, this is first 'body' part in the msg (after headers) */
		if (attachCounter == 1)
		{
			/* Let's take the headers from the message */
			/* We position to the beginning of the stream (message) */
			if ((lseek(0, 0, SEEK_SET)) == (off_t)-1)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to positioned at the beginning of the stream (stdin) using lseek()");
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] positioned at the beginning of the stream (stdin) using lseek()");
			}
		
			/*
			  We allocate 'start+1' for readData
			  Basically, what we do is:
			    - position to 0 (beginning of the stream)
				- get the 'start' of 2nd MIME part (actually body message)
				- assume that 0 -> start contains headers (yet to be proven 100% :)
				- get that, and also get the 'boundary' by "rewinding" to the beginning of the previous line and grabbing that line (and cutting '\n')
			*/
		
			/* Do I need that +1? */
			if ((readData = calloc(1, start+1)) == NULL)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to calloc() ram for [%lu] bytes", (unsigned long)start+1);
				/* FIXME - we should probably isolate the mail and scream! However, mail that makes this calloc() fails is probably 'naughty' one, so... do we care? :) */
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] calloc()ed [%lu] bytes", (unsigned long)start+1);
			}
		
			/* Read the data (headers) into readData buffer */
			if ((sizeRead = read(0, readData, start)) == -1)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to read() [%lu] bytes into readData", (unsigned long)start);
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] read() [%lu] bytes into readData", sizeRead);
			}

			if ((sizeWritten = write(rfd, readData, start)) == -1)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to write() [%lu] bytes into rfd filehandle", (unsigned long)start+1);
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] managed to write() [%lu] bytes into rfd filehandle", sizeWritten);
			}
			
			/* Free the allocated memory */
			free(readData);
			virge_debug(3, "[ rewrite_attachments() ] free()ed readData (hopefully ;)");
	
			/* Now, we get the boundary - very stupid way, but might actualy work in 100% of cases - depends on librfc2045 */
			/* Set 'cur' to start-2 (so that we skip '\n' of previous line) */
			virge_debug(3, "[ rewrite_attachments() ] about to start 'rewinding' back in order to get boundary");
			cur = (unsigned long)start-2;
			do
			{
				fseek(stdin, cur--, SEEK_SET);
			} while (fgetc(stdin) != '\n');

			virge_debug(3, "[ rewrite_attachments() ] seems like we have found the beginning of boundary line");

			/* Boundary can't be more than 4095... if it is, then message probably won't be important anyway ;) */
			fgets(boundary, 4095, stdin);

			/* Strip the newline */
			if (strchr(boundary, '\n'))
				*strchr(boundary, '\n') = 0;

			virge_debug(3, "[ rewrite_attachments() ] boundary is: [%s] (hope it's right :)", boundary);
		}
	
		/* FIXME - damn, rescanning all over again just to find out why we are rewriting?! FIXME */
		if (disposition_filename && *disposition_filename)
		{
			if (check_extension_regex((char *)disposition_filename))
			{
				skipFile = 1;
				virge_debug(3, "[ rewrite_attachments() ] rewriting message because filename '%s' is not allowed", disposition_filename);
			}
		}

		if (disposition_name && *disposition_name)
		{
			if (check_extension_regex((char *)disposition_name))
			{
				skipFile = 2;
				virge_debug(3, "[ rewrite_attachments() ] rewriting message because filename '%s' is not allowed", disposition_name);
			}
		}
	
		if (content_name && *content_name)
		{
			if (check_extension_regex((char *)content_name))
			{
				skipFile = 3;
				virge_debug(3, "[ rewrite_attachments() ] rewriting message because filename '%s' is not allowed", content_name);
			}
		}

		/* Do we 'skip' (not send to recipient) this part or not? */
		if (skipFile < 1)
		{
			/* Ok, include this section into the mail/message */
			long readLines = (unsigned long)nlines;
			static char readLine[V_BUFSIZE];
			unsigned long boundaryWritten;

			virge_debug(3, "[ rewrite_attachments() ] about to read [%lu] lines", readLines);

			if ((fseek(stdin, start, SEEK_SET)) == -1)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to position at the start of the MIME section (position: [%lu]) using fseek()", (unsigned long)start);
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] positioned at the start of the MIME section (position: [%lu]) using fseek()", (unsigned long)start);
			}
			
			while(readLines >= 0)
			{
				if (!fgets(readLine, V_BUFSIZE-1, stdin))
				{
					virge_debug(2, "[ rewrite_attachments() ] failed to get a line using fgets()");
					virge_syslog("FAILURE: fgets() did not receive line in rewrite_attachments(), for Message ID: '%s'", messageID);
				}
				else
				{
					/* If first line read from MIME section is empty - skip it (it wouldn't be RFC compliant if boundary is followed by empty line */
//					if ( (readLines == nlines) && (readLine[0] == '\n') )
//					{
//						/* Skip? To see later! */
//					}
//					else
//					{
						write(rfd, readLine, strlen(readLine));
//					}

					readLines = readLines - 1;
				}
			}

			/* Add boundary */
			snprintf(boundaryString, sizeof(boundaryString)-1, "\n%s\n", boundary);
			if ((boundaryWritten = write(rfd, boundaryString, strlen(boundaryString))) == -1)
			{
				virge_debug(2, "[ rewrite_attachments() ] failed to write() [%lu] bytes (boundaryString) to rfd filehandle", strlen(boundaryString));
			}
			else
			{
				virge_debug(3, "[ rewrite_attachments() ] managed to write() [%lu] bytes (boundaryString) to rfd filehandle", boundaryWritten);
			}
		}
		else
		{
			switch (skipFile)
			{
				/* disposition_filename */
				case 1:
				strncpy(REMOVED[removedCount], disposition_filename, CONFIG_LEN-1);
				virge_debug(3, "[ rewrite_attachments() ] added '%s' [disposition_filename] to REMOVED array", disposition_filename);
				break;
				
				case 2:
				strncpy(REMOVED[removedCount], disposition_name, CONFIG_LEN-1);
				virge_debug(3, "[ rewrite_attachments() ] added '%s' [disposition_name] to REMOVED array", disposition_name);
				break;

				case 3:
				strncpy(REMOVED[removedCount], content_name, CONFIG_LEN-1);
				virge_debug(3, "[ rewrite_attachments() ] added '%s' [content_name] to REMOVED array", content_name);
				break;
			}

			removedCount++;
		}
	}

	/* attachCounter is just used to 'count' the number of MIME sections in the message - rfc2045_decode() invokes check_attachment() multiple times */
	attachCounter++;
}

void removed_attachments(char *removedList)
{
	int next = 0;
	int remPos;

	if (removedCount > 0)
	{
		strncpy(removedList, "REMOVED    : ", 13);

		virge_debug(3, "[ removed_attachments() ] Going to create string w/ list of removed attachments");

		for (remPos = 0 ; remPos < removedCount ; remPos++)
		{
			virge_debug(3, "[ removed_attachments() ] Processing : [%s]", REMOVED[remPos]);

			if (next > 0)
			{
				strncat(removedList, "           : ", 13);
				strncat(removedList, "\"", 1);
				strncat(removedList, REMOVED[remPos], 128);
				strncat(removedList, "\"\n", 2);
				
				next++;
				
				/* If I am not wrong, we could overflow V_LOGMSG if too many strncat()'s are made */
				/* So, we'll just return after 25 loops */
				if (next > 25)
					return;
			}
			else
			{
				strncat(removedList, "\"", 1);
				strncat(removedList, REMOVED[remPos], 128);
				strncat(removedList, "\"\n", 2);
				next++;
			}
		}
	}
	else
	{
		virge_debug(2, "[ removed_attachments() ] We have been invoked, but seems that no attachments were removed. A bug!?");
	}
}
