#include "virge.h"

/*
  Reads/loads extensions from V_EXT file (by default is /virge/etc/extensions) - attachments are later checked
  against those extensions (could be full filenames, if you wish).
*/
int load_extensions(void)
{
	FILE *fext;
	char extension[CONFIG_LEN];
	int extCount = 0;
	
	if (!(fext = fopen(V_EXT, "r")))
	{
		virge_debug(2, "[ load_extensions() ] failed to open 'extensions' file '%s'", V_EXT);
		virge_syslog("FAILURE: Could not read extensions configuration file '%s'", V_EXT);
		return(0);
	}

	while (fgets(extension, CONFIG_LEN-1, fext))
	{
		/* Process if it's not empty line ('\n') or comment (begins with '#') */
		if ( (extension[0] != '\n') && (extension[0] != '#') )
		{
			if (strchr(extension, '\n'))
				*strchr(extension, '\n') = 0;

			/*
			  Some people might want to have comments for extensions
			  Let's try to make it possible
			*/
			
			/*
			  Small explanation:
			  
			  If you have extension line like:
			  
			  "\.exe$ 	# crap" (1 space, 1 tab and 1 '#' after the extension)
			  
			  it will be stripped to "\.exe$"
			*/

			/* 'Cut' the extension at first occurence of tab character */
			if (strchr(extension, '\t'))
				*strchr(extension, '\t') = '\0';
			/*
			  If I am not wrong, '#' is not used in regexps - if it is, let me know
			  Also, if *extension* (or filename in extensions file) contains '#', it'll be screwed
			*/
			if (strchr(extension, '#'))
				*strchr(extension, '#') = '\0';
			
			/* Now, if there is also a space behind the extension - cut there */
			if (strchr(extension, ' '))
				*strchr(extension, ' ') = '\0';

			strncpy(EXTENSIONS[extCount++], extension, CONFIG_LEN-1);
			virge_debug(3, "[ load_extensions() ] added extension '%s'", extension);
		}
	}

	fclose(fext);

	virge_debug(1, "[ load_extensions() ] [%d] extensions loaded", extCount);
	return(extCount);
}

/*
  Reads/loads list of users for which mails should be delivered immediately (no checks made)
*/
int load_allowusers(void)
{
	FILE *fusers;
	char allowUser[CONFIG_LEN];
	
	if (!(fusers = fopen(V_USERS, "r")))
	{
		virge_debug(2, "[ load_allowusers() ] failed to open 'allowed users' file '%s'", V_USERS);
		virge_syslog("FAILURE: Could not read 'allowed users' configuration file '%s'", V_USERS);
		return(0);
	}

	while (fgets(allowUser, CONFIG_LEN-1, fusers))
	{
		/* Process if it's not empty line ('\n') */
		if (allowUser[0] != '\n')
		{
			if (strchr(allowUser, '\n'))
				*strchr(allowUser, '\n') = 0;

			strncpy(ALLOW_USERS[allowUserCount++], allowUser, CONFIG_LEN-1);
			virge_debug(3, "[ load_allowusers() ] added user '%s'", allowUser);
		}
	}

	fclose(fusers);

	virge_debug(1, "[ load_allowusers() ] loaded [%d] users", allowUserCount);
	return(allowUserCount);
}

/* Sets configuration parameters */
int config_set(char *confLine, char *confEntry, char *configData)
{
	int confPos;
	int c;
	
	if (!CONFIG_CMP(confLine, confEntry))
	{
		confPos = strlen(confEntry);
		
		for(c = confPos; c < strlen(confLine); c++)
		{
			if ( (confLine[c] != '\t') && (confLine[c] != ' ') )
			{
				break;
			}
		}

		if (strlen(&confLine[c]) < 1)
		{
			virge_debug(2, "[ config_set() ] configuration parameter '%s' has invalid (empty) value - aborting! [offending line: '%s']", confEntry, confLine);
			virge_syslog("FAILURE: Configuration parameter '%s' has invalid/empty value [%s]", confEntry, confLine);
			fprintf(stderr, "ERROR: Configuration parameter '%s' invalid [%s] - aborting!\n", confEntry, confLine);
			exit(1);
		}

		strncpy(configData, &confLine[c], 255);
		virge_debug(1, "[ config_set() ] configuration parameter '%s' set: [%s]", confEntry, configData);
		return(1);
	}
	
	return(0);
}

/* Load configuration from a file */
void load_virge_config(void)
{
	FILE *fconfig;
		
	char configLine[V_ARGSIZE];
	char confData[V_ARGSIZE];

	/* Open the file first */
	if ((fconfig = fopen(V_CONFIG, "r")))
	{
		while(!feof(fconfig))
		{
			fgets(configLine, V_ARGSIZE-1, fconfig);
			if ( (configLine[0] != '\n') && (configLine[0] != '#') )
			{
				if (strchr(configLine, '\n'))
					*strchr(configLine, '\n') = 0;
	
				confData[0] = 0;

				/* virge_dir: */
				if (config_set(configLine, "virge_dir:", confData))
					strncpy(V_DIR, confData, sizeof(V_DIR)-1);

				/* log_dir: */
				else if (config_set(configLine, "log_dir:", confData))
					if (confData[0] == '/')
						strncpy(V_LOGDIR, confData, sizeof(V_LOGDIR)-1);
					else
						snprintf(V_LOGDIR, sizeof(V_LOGDIR)-1, "%s/%s", V_DIR, confData);

				/* log_virge: */
				else if (config_set(configLine, "log_virge:", confData))
					if (confData[0] == '/')
						strncpy(LOG_VIRGE, confData, sizeof(LOG_VIRGE)-1);
					else
						snprintf(LOG_VIRGE, sizeof(LOG_VIRGE)-1, "%s/%s", V_LOGDIR, confData);

				/* log_error: */
				else if (config_set(configLine, "log_error:", confData))
					if (confData[0] == '/')
						strncpy(LOG_ERROR, confData, sizeof(LOG_ERROR)-1);
					else
						snprintf(LOG_ERROR, sizeof(LOG_ERROR)-1, "%s/%s", V_LOGDIR, confData);

				/* log_action: */
				else if (config_set(configLine, "log_action:", confData))
					if (confData[0] == '/')
						strncpy(LOG_ACTION, confData, sizeof(LOG_ACTION)-1);
					else
						snprintf(LOG_ACTION, sizeof(LOG_ACTION)-1, "%s/%s", V_LOGDIR, confData);

				/* log_virus: */
				else if (config_set(configLine, "log_virus:", confData))
					if (confData[0] == '/')
						strncpy(LOG_VIRUS, confData, sizeof(LOG_VIRUS)-1);
					else
						snprintf(LOG_VIRUS, sizeof(LOG_VIRUS)-1, "%s/%s", V_LOGDIR, confData);

				/* isolate_dir: */
				else if (config_set(configLine, "isolate_dir:", confData))
					if (confData[0] == '/')
						strncpy(V_ISODIR, confData, sizeof(V_ISODIR)-1);
					else
						snprintf(V_ISODIR, sizeof(V_ISODIR)-1, "%s/%s", V_DIR, confData);

				/* etc_dir: */
				else if (config_set(configLine, "etc_dir:", confData))
					if (confData[0] == '/')
						strncpy(V_ETCDIR, confData, sizeof(V_ETCDIR)-1);
					else
						snprintf(V_ETCDIR, sizeof(V_ETCDIR)-1, "%s/%s", V_DIR, confData);

				/* temp_dir: */
				else if (config_set(configLine, "temp_dir:", confData))
					if (confData[0] == '/')
						strncpy(V_TEMPDIR, confData, sizeof(V_TEMPDIR)-1);
					else
						snprintf(V_TEMPDIR, sizeof(V_TEMPDIR)-1, "%s/%s", V_DIR, confData);

				/* runas_user: */
				else if (config_set(configLine, "runas_user:", confData))
					strncpy(RUNAS_USER, confData, sizeof(RUNAS_USER)-1);

				/* virge_dir_mode: */
				else if (config_set(configLine, "virge_dir_mode:", confData))
					strncpy(VIRGE_DIR_MODE, confData, sizeof(VIRGE_DIR_MODE)-1);

				/* isolated_mode: */
				else if (config_set(configLine, "isolated_mode:", confData))
					strncpy(ISOLATED_FILE_MODE, confData, sizeof(ISOLATED_FILE_MODE)-1);

				/* procmail: */
				else if (config_set(configLine, "procmail:", confData))
					strncpy(PROCMAIL, confData, sizeof(PROCMAIL)-1);

				/* sendmail: */
				else if (config_set(configLine, "sendmail:", confData))
					strncpy(SENDMAIL, confData, sizeof(SENDMAIL)-1);

				/* admin: */
				else if (config_set(configLine, "admin:", confData))
					strncpy(ADMIN, confData, sizeof(ADMIN)-1);

				/* notify_use_header_rcpt: */
				else if (config_set(configLine, "notify_use_header_rcpt:", confData))
				{
					if (isdigit((int) *confData))
						NOTIFY_USE_HEADER_RCPT = atoi(confData);
				}

				/* notify_sender_virus: */
				else if (config_set(configLine, "notify_sender_virus:", confData))
				{
					if (isdigit((int) *confData))
						NOTIFY_SENDER_VIRUS = atoi(confData);
				}

				/* notify_sender_attach: */
				else if (config_set(configLine, "notify_sender_attachment:", confData))
				{
					if (isdigit((int) *confData))
						NOTIFY_SENDER_ATTACHMENT = atoi(confData);
				}

				/* virge_from: */
				else if (config_set(configLine, "virge_from:", confData))
					strncpy(VIRGE_FROM, confData, sizeof(VIRGE_FROM)-1);

				/* extensions_file: */
				else if (config_set(configLine, "extensions_file:", confData))
					if (confData[0] == '/')
						strncpy(V_EXT, confData, sizeof(V_EXT)-1);
					else
						snprintf(V_EXT, sizeof(V_EXT)-1, "%s/%s", V_ETCDIR, confData);

				/* allowusers_file: */
				else if (config_set(configLine, "allowusers_file:", confData))
					if (confData[0] == '/')
						strncpy(V_USERS, confData, sizeof(V_USERS)-1);
					else
						snprintf(V_USERS, sizeof(V_USERS)-1, "%s/%s", V_ETCDIR, confData);

				/* mail_body: */
				else if (config_set(configLine, "mail_body:", confData))
					strncpy(V_MAILBODY, confData, sizeof(V_MAILBODY)-1);

				/* deliver_loadavg: */
				else if (config_set(configLine, "deliver_loadavg:", confData))
				{
					if (isdigit((int) *confData))
						V_DELIVER_LOADAVG = atoi(confData);
				}

				/* scan_avp: */
				else if (config_set(configLine, "scan_avp:", confData))
				{
					if (confData[0] == '0')
						V_SCAN_AVP = 0;
				}

				/* scan_sophie: */
				else if (config_set(configLine, "scan_sophie:", confData))
				{
					if (confData[0] == '0')
						V_SCAN_SOPHIE = 0;
				}

				/* scan_sophie_socket: */
				else if (config_set(configLine, "scan_sophie_socket:", confData))
					strncpy(V_SCAN_SOPHIE_SOCKET, confData, sizeof(V_SCAN_SOPHIE_SOCKET)-1);

				/* virge_timeout: */
				else if (config_set(configLine, "virge_timeout:", confData))
				{
					if (isdigit((int) *confData))
						VIRGE_TIMEOUT = atoi(confData);
				}

				/* debug: */
				else if (config_set(configLine, "debug:", confData))
				{
					if (!CMDLINE_DEBUG)
						if (isdigit((int) *confData))
							DEBUG = atoi(confData);
				}

				/* debug_level: */
				else if (config_set(configLine, "debug_level:", confData))
				{
					if (!CMDLINE_DEBUG_LEVEL)
						if (isdigit((int) *confData))
							DEBUG_LEVEL = atoi(confData);
				}
			}
		}
	}
	else
	{
		virge_debug(2, "[ load_virge_config() ] Configuration file not found! [%s] - aborting!", V_CONFIG);
		fprintf(stderr, "ERROR: Configuration file not found [%s] - aborting!\n", V_CONFIG);
		exit(1);
	}
}

/*
  Check the validity of current (loaded) configuration
  The 'checks' are pretty simple - don't rely on it!
*/
void check_current_config(void)
{
	/* I might add more checks later (permissions on dirs, files, etc) */

	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "-- CONFIGURATION CHECK INITIATED\n\n");

	/* Does RUNAS_USER exist? */
	if (!getpwnam(RUNAS_USER))
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] RUNAS_USER [%s] does not exist!\n", RUNAS_USER);

		virge_debug(2, "[ check_current_config() ] Invalid RUNAS_USER [%s]", RUNAS_USER);
		fprintf(stderr, "ERROR: Invalid RUNAS_USER [%s] - aborting!\n", RUNAS_USER);
		exit(1);
	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] RUNAS_USER [%s] exists!\n", RUNAS_USER);
	}

	/* Does PROCMAIL exist? */
	if (!file_exists(PROCMAIL))
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Procmail [%s] does not exist!\n", PROCMAIL);
		
		virge_debug(2, "[ check_current_config() ] Invalid PROCMAIL [%s]", PROCMAIL);
		fprintf(stderr, "ERROR: Invalid PROCMAIL settings [%s] - aborting!\n", PROCMAIL);
		exit(1);
	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [   OK] Procmail [%s] exists!\n", PROCMAIL);
	}

	/* Does SENDMAIL exist? */
	if (!file_exists(SENDMAIL))
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Sendmail [%s] does not exist!\n", SENDMAIL);

		virge_debug(2, "[ check_current_config() ] Invalid SENDMAIL [%s]", SENDMAIL);
		fprintf(stderr, "ERROR: Invalid SENDMAIL settings [%s] - aborting!\n", SENDMAIL);
		exit(1);

	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] Sendmail [%s] exists!\n", SENDMAIL);
	}

	/* Is ADMIN specified? */
	if (strlen(ADMIN) < 1)
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] ADMIN email is not specified\n");
		
		virge_debug(2, "[ check_current_config() ] ADMIN not specified");
		fprintf(stderr, "ERROR: ADMIN not specified - aborting\n");
		exit(1);
	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [   OK] ADMIN email [%s] is specified!\n", ADMIN);
	}

	/* Is VIRGE_FROM specified? */
	if (strlen(VIRGE_FROM) < 1)
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] VIRGE_FROM email is not specified!\n");
		
		virge_debug(2, "[ check_current_config() ] VIRGE_FROM not specified");
		fprintf(stderr, "ERROR: VIRGE_FROM not specified - aborting!\n");
		exit(1);
	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [   OK] VIRGE_FROM email [%s] is specified!\n", VIRGE_FROM);
	}

	/* V_DIR check */
	/* We try to create a dir (if it doesn't exist) */
	check_req_dir(V_DIR, "ROOT");
	if (CHECK_CONFIG_VERBOSE)
	{
		if (!file_exists(V_DIR))
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Main Virge directory [%s] does not exist!\n", V_DIR);
		else
			fprintf(stdout, "CONFIG_CHECK: [   OK] Main Virge directory [%s] exists\n", V_DIR);
	}

	/* V_LOGDIR check */
	check_req_dir(V_LOGDIR, "LOG");
	if (CHECK_CONFIG_VERBOSE)
	{
		if (!file_exists(V_LOGDIR))
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Virge log directory [%s] does not exist!\n", V_LOGDIR);
		else
			fprintf(stdout, "CONFIG_CHECK: [   OK] Virge log directory [%s] exists!\n", V_LOGDIR);
	}

		/* Does LOG_VIRGE file exist? */
		if (CHECK_CONFIG_VERBOSE)
		{
			if (!file_exists(LOG_VIRGE))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] LOG_VIRGE file [%s] does not exist!\n", LOG_VIRGE);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] LOG_VIRGE file [%s] can/will be created later!\n", LOG_VIRGE);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] LOG_VIRGE file [%s] exists!\n", LOG_VIRGE);
		}

		/* Does LOG_ERROR file exist? */
		if (CHECK_CONFIG_VERBOSE)
		{
			if (!file_exists(LOG_ERROR))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] LOG_ERROR file [%s] does not exist!\n", LOG_ERROR);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] LOG_ERROR file [%s] can/will be created later!\n", LOG_ERROR);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] LOG_ERROR file [%s] exists!\n", LOG_ERROR);
		}

		if (CHECK_CONFIG_VERBOSE)
		{
		/* Does LOG_ACTION file exist? */
			if (!file_exists(LOG_ACTION))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] LOG_ACTION file [%s] does not exist!\n", LOG_ACTION);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] LOG_ACTION file [%s] can/will be created later!\n", LOG_ACTION);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] LOG_ACTION file [%s] exists!\n", LOG_ACTION);
		}

		if (CHECK_CONFIG_VERBOSE)
		{
		/* Does LOG_VIRUS file exist? */
			if (!file_exists(LOG_VIRUS))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] LOG_VIRUS file [%s] does not exist!\n", LOG_VIRUS);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] LOG_VIRUS file [%s] can/will be created later!\n", LOG_VIRUS);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] LOG_VIRUS file [%s] exists!\n", LOG_VIRUS);
		}

	/* V_TEMPDIR check */
	check_req_dir(V_TEMPDIR, "TEMP");
	if (CHECK_CONFIG_VERBOSE)
	{
		if (!file_exists(V_TEMPDIR))
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Virge temp directory [%s] does not exist!\n", V_TEMPDIR);
		else
			fprintf(stdout, "CONFIG_CHECK: [   OK] Virge temp directory [%s] exists!\n", V_TEMPDIR);
	}

	/* V_ISODIR check */
	check_req_dir(V_ISODIR, "ISOLATION");
	if (CHECK_CONFIG_VERBOSE)
	{
		if (!file_exists(V_ISODIR))
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Virge isolation directory [%s] does not exist!\n", V_ISODIR);
		else
			fprintf(stdout, "CONFIG_CHECK: [   OK] Virge isolation directory [%s] exists!\n", V_ISODIR);
	}

	/* V_ETCDIR check */
	check_req_dir(V_ETCDIR, "ETC");
	if (CHECK_CONFIG_VERBOSE)
	{
		if (!file_exists(V_ETCDIR))
			fprintf(stdout, "CONFIG_CHECK: [ERROR] Virge etc (config) directory [%s] does not exist!\n", V_ETCDIR);
		else
			fprintf(stdout, "CONFIG_CHECK: [   OK] Virge etc (config) directory [%s] exists!\n", V_ETCDIR);
	}

		/* Does V_EXT file exist? */
		if (CHECK_CONFIG_VERBOSE)
		{
			if (!file_exists(V_EXT))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] V_EXT file [%s] does not exist!\n", V_EXT);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] V_EXT file [%s] can be created later!\n", V_EXT);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] V_EXT file [%s] exists!\n", V_EXT);
		}

		/* Does V_USERS file exist? */
		if (CHECK_CONFIG_VERBOSE)
		{
			if (!file_exists(V_USERS))
			{
				fprintf(stdout, "CONFIG_CHECK: [ERROR] V_USERS file [%s] does not exist!\n", V_USERS);
				fprintf(stdout, "CONFIG_CHECK: [ NOTE] V_USERS file [%s] can be created later!\n", V_USERS);
			}
			else
				fprintf(stdout, "CONFIG_CHECK: [   OK] V_USERS file [%s] exists!\n", V_USERS);
		}

	/* NOTIFY_USE_HEADER_RCPT value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] NOTIFY_USE_HEADER_RCPT setting: [%d]\n", NOTIFY_USE_HEADER_RCPT);

	/* NOTIFY_SENDER_VIRUS value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] NOTIFY_SENDER_VIRUS setting: [%d]\n", NOTIFY_SENDER_VIRUS);

	/* NOTIFY_SENDER_ATTACHMENT value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] NOTIFY_SENDER_ATTACHMENT setting: [%d]\n", NOTIFY_SENDER_ATTACHMENT);

	/* DIR_MODE value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] VIRGE_DIR_MODE setting: [%s]\n", VIRGE_DIR_MODE);

	/* ISOLATED_FILE_MODE value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] ISOLATED_FILE_MODE setting: [%s]\n", ISOLATED_FILE_MODE);

	/* V_SCAN_AVP value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] V_SCAN_AVP setting: [%d]\n", V_SCAN_AVP);

	/* V_SCAN_SOPHIE value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] V_SCAN_SOPHIE setting: [%d]\n", V_SCAN_SOPHIE);

	/* V_SCAN_SOPHIE_SOCKET value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] V_SCAN_SOPHIE_SOCKET: [%s]\n", V_SCAN_SOPHIE_SOCKET);

	/* V_SCAN_SOPHIE_SOCKET value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] VIRGE_TIMEOUT: [%d]\n", VIRGE_TIMEOUT);

	/* V_MAILBODY */
	if (strlen(V_MAILBODY) < 1)
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ERROR] V_MAILBODY is not set\n");
		
		/* Set the default then */
		strncpy(V_MAILBODY, "mail.body", sizeof(V_MAILBODY)-1);
		virge_debug(1, "[ check_current_config() ] V_MAILBODY set to 'mail.body'");

		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [ NOTE] V_MAILBODY is now set to 'mail.body'\n");
	}
	else
	{
		if (CHECK_CONFIG_VERBOSE)
			fprintf(stdout, "CONFIG_CHECK: [   OK] V_MAILBODY is set to [%s]\n", V_MAILBODY);
	}

	/* V_DELIVER_LOADAVG value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [   OK] V_DELIVER_LOADAVG setting: [%d]\n", V_DELIVER_LOADAVG);

	/* DEBUG value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [ NOTE] DEBUG set to [%d]\n", DEBUG);

	/* DEBUG_LEVEL value */
	if (CHECK_CONFIG_VERBOSE)
		fprintf(stdout, "CONFIG_CHECK: [ NOTE] DEBUG_LEVEL set to [%d]\n", DEBUG_LEVEL);

	if (CHECK_CONFIG_VERBOSE)
	{
		fprintf(stdout, "\n-- CONFIGURATION CHECK COMPLETE\n");
		exit(0);
	}

}

void check_regex(void)
{
	int eCount;
	int regCur;
	int regErr;
	
	regex_t t_preg;
	
	/* Load the extensions */
	eCount = load_extensions();

	for (regCur = 0 ; regCur < eCount ; regCur++)
	{
		if ((regErr = regcomp(&t_preg, EXTENSIONS[regCur], REG_ICASE)))
		{
			/* 0 was not returned - something was wrong */
			switch (regErr)
			{
				case REG_BADRPT:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_BADRPT);
				break;
				
				case REG_BADBR:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_BADBR);
				break;
				
				case REG_EBRACE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_EBRACE);
				break;
				
				case REG_EBRACK:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_EBRACK);
				break;
				
				case REG_ERANGE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ERANGE);
				break;
				
				case REG_ECTYPE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ECTYPE);
				break;
				
				case REG_ECOLLATE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ECOLLATE);
				break;
				
				case REG_EPAREN:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_EPAREN);
				break;
				
				case REG_ESUBREG:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ESUBREG);
				break;

/* Not in *BSD */
#ifdef LINUX
				case REG_EEND:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_EEND);
				break;
#endif				
				case REG_EESCAPE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_EESCAPE);
				break;
				
				case REG_BADPAT:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_BADPAT);
				break;
/* Not in *BSD */
#ifdef LINUX
				case REG_ESIZE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ESIZE);
				break;
#endif
				case REG_ESPACE:
				print_regex_err(EXTENSIONS[regCur], ERR_REG_ESPACE);
				break;
				
				default:
				print_regex_err(EXTENSIONS[regCur], "UNKNOWN ERROR");
				break;
			}
		}
		else
		{
			fprintf(stdout, "[ OK ] (%s)\n", EXTENSIONS[regCur]);
		}

		regfree(&t_preg);
	}
}

void print_regex_err(char *regex_test, char *regex_err_msg)
{
	fprintf(stdout, "[FAIL] (%s) \"%s\"\n", regex_test, regex_err_msg);
}
