#include "virge.h"

/*
  Sends debug messages to stderr or syslog (1 = stderr, 2 = syslog)
*/  
void virge_debug(int debugLevel, char *debugMessage, ...)
{
	char debugMsg[V_LOGMSG];

	va_list argptr;
	va_start(argptr, debugMessage);
	vsnprintf(debugMsg, sizeof(debugMsg)-1, debugMessage, argptr);
	va_end(argptr);

	if (debugLevel <= DEBUG_LEVEL)
	{
		if (DEBUG == 1)
		{
			fprintf(stderr, "[%s] [%d] debug: %s\n", messageID[0] ? messageID : "(not set yet!)", debugLevel, debugMsg);
		}
		else if (DEBUG == 2)
		{
			openlog(V_PROGNAME, LOG_PID | LOG_CONS, LOG_USER);
			syslog(LOG_NOTICE, "[%s] [%d] debug: %s\n", messageID[0] ? messageID : "(not set yet!)", debugLevel, debugMsg);
			closelog();
		}
	}
}

/*
  Logs message to virge logfile
  
  logPath = LOG_VIRGE / LOG_ERROR / LOG_ACTION
  logMessage = formatted string (message to be logged)
  
  Example:
  
  event_log(LOG_ACTION, "User '%s' is probably spamming user '%s' - remember to crash their mail server", MAIL_FROM, RCPT_TO);
*/
int event_log(char *logPath, char *logMessage, ...)
{
	char logString[V_LOGMSG];
	char logFinal[V_LOGMSG];
	int logFile;

	va_list argptr;
	va_start(argptr, logMessage);
	vsnprintf(logString, sizeof(logString)-1, logMessage, argptr);
	va_end(argptr);

	snprintf(logFinal, V_LOGMSG-1, "%s: %s\n", messageID, logString);

	/* Log to syslog if file can't be opened... */
	if ((logFile = open(logPath, O_CREAT | O_RDWR | O_NOFOLLOW | O_APPEND, 0600)) == -1)
	{
		virge_debug(2, "[ event_log() ] failed to open logfile (%s)", logPath);
		virge_syslog("FAILURE: Could not open logfile '%s' for writing", logPath);
	}
	else
	{
		write(logFile, logFinal, strlen(logFinal));
		close(logFile);
	}

	return 0;
}

/*
  Logs a message via syslog()
  
  syslogMessage = formatted string (message to be logged)
  
  Example:
  
  virge_syslog("Some idiot called '%s' just told us how to get rich in 2 weeks. WOW!", MAIL_FROM);
*/
void virge_syslog(char *syslogMessage, ...)
{
	char logMsg[V_LOGMSG];

	va_list argptr;
	va_start(argptr, syslogMessage);
	vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
	va_end(argptr);

	openlog(V_PROGNAME, LOG_PID | LOG_CONS, LOG_USER);
	syslog(LOG_NOTICE, "%s\n", logMsg);
	closelog();
}
