#include "virge.h"

/*
  Deliver mails through procmail
  
  Content of mailFile is sent to procmail (through pipe) for local delivery
  
  mailFile = file that contains message
  aParam = -a param, as received from Sendmail (have absolutelly no idea what it's used for - might check it out one day :)
  direct_delivery = 'Flag' that tells if mail has been delivered 'directly' (no checking for viruses/attachments) or not. Have absolutelly no idea why I thought I'd need it...

  Example:
  
  virge_procmail(tempfile, arg_a, 1);
*/
void virge_procmail(char *mailFile, char *aParam, int direct_delivery)
{
	char sendbuf[V_BUFSIZE];
	char procArgs[V_MSGSIZE]; /* just because of the size (email + recipient + etc.) - we'll maybe #define other vars, doh... FIXME */
	int bread;
	int mailfd;
	FILE *procmail;

	if (!(mailfd = open(mailFile, O_RDONLY | O_NOFOLLOW)))
	{
		virge_debug(2, "[ virge_procmail() ] failed to open mailfile (%s)", mailFile);
		virge_syslog("FAILURE: Error occured while open()-ing mail file in virge_procmail()");
	}
	else
	{
		virge_debug(3, "[ virge_procmail() ] Mailfile (message) to be delivered (%s) - open()ed", mailFile);

		/* Create a string to be passed to popen() - delivering mail through procmail */
		/* bloody 'aParam' - FIXME later */
		if (strlen(aParam) < 1)
			snprintf(procArgs, sizeof(procArgs)-1, "%s -Y -f %s -a '' -d %s", PROCMAIL, MAIL_FROM, RCPT_TO);
		else
			snprintf(procArgs, sizeof(procArgs)-1, "%s -Y -f %s -a %s -d %s", PROCMAIL, MAIL_FROM, aParam, RCPT_TO);
	
		virge_debug(3, "[ virge_procmail() ] procArgs string = [%s]", procArgs);

		/* Add error checking - possibly, add some kind of 'failure queue', where failed mails (which couldn't be delivered
		   because of problem with popen() ) could be stored. Or something...  TODO */
		/* Bollocks - can't figure out how to catch 'errors' if popen() fails - NULL doesn't get caught... */
		procmail = popen(procArgs, "w");
		virge_debug(3, "[ virge_procmail() ] popen() created pipe to procmail");

		/* Read the maildata and pass it to procmail */
		/* use strlen(sendbuf) here? sizeof(sendbuf) will always be V_BUFSIZE, no? */
		while ( (bread = read(mailfd, sendbuf, sizeof(sendbuf))) > 0)
		{
			if (!fwrite(sendbuf, bread, 1, procmail))
			{
				if (ferror(procmail) != 0)
				{
					virge_debug(2, "[ virge_procmail() ] error during write()");
					virge_syslog("FAILURE: Error occured during fwrite() in virge_procmail()");
					break;
				}
			}
		}

		pclose(procmail);
		close(mailfd);
		virge_debug(3, "[ virge_procmail() ] procmail pipe closed, as is mailfd descriptor");
		
		/*
		  [-] = delivered immediately/directly - no checking
		  [+] = delivered after checks have been performed
		  [!] = delivered after message has been rewritten
		*/
		if (direct_delivery == 1)
		{
			event_log(LOG_VIRGE, "Mail delivered [-] from=<%s>, to=<%s>, size=<%lu bytes>", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
		}
		else
		{
			if (REWRITE == 0)
				event_log(LOG_VIRGE, "Mail delivered [+] from=<%s>, to=<%s>, size=<%lu bytes>", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
			else
				event_log(LOG_VIRGE, "Mail delivered [!] from=<%s>, to=<%s>, size=<%lu bytes>", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);
		}

		virge_debug(1, "[ virge_procmail() ] mail delivered from '%s' to '%s' - size: [%lu bytes]", MAIL_FROM, RCPT_TO, V_MAIL_SIZE);

	}
}

/*
  Replacement for sendmail.org's Sendmail product (you should be excited, no?)
  
  email = email address of the loser who is wasting the bandwidth (probably admin)
  sendMessage = text/body of the message to be sent
  
  Example:
  
  virge_sendmail("rewt@/dev/null", "See you tonight. Love you. /dev/zero.");
*/
void virge_sendmail(char *email, char *sendMessage, char *sendSubject)
{
	char sendMsg[V_MSGSIZE];
	char sendmailInvoke[V_FILESIZE];
	FILE *sendmail;
	
	snprintf(sendmailInvoke, sizeof(sendmailInvoke)-1, "%s -f %s -t", SENDMAIL, VIRGE_FROM);
	virge_debug(3, "[ virge_sendmail() ] Going to invoke sendmail as '%s'", sendmailInvoke);

	sendmail = popen(sendmailInvoke, "w");
	virge_debug(3, "[ virge_sendmail() ] pipe to sendmail opened");
	
	snprintf(sendMsg, sizeof(sendMsg)-1, "From: Virge <%s>\n", VIRGE_FROM);
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 1st fwrite()");
			virge_syslog("FAILURE: Error occured during 1st fwrite() in virge_sendmail()");
			/* Should we exit/return/whatever in here? FIXME */
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	snprintf(sendMsg, sizeof(sendMsg)-1, "To: %s\n", email);
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 2nd fwrite()");
			virge_syslog("FAILURE: Error occured during 2nd fwrite() in virge_sendmail()");
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	snprintf(sendMsg, sizeof(sendMsg)-1, "Subject: [VIRGE] %s\n", sendSubject);
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 3rd fwrite()");
			virge_syslog("FAILURE: Error occured during 3rd fwrite() in virge_sendmail()");
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	snprintf(sendMsg, sizeof(sendMsg)-1, "X-VirgeMsgID: %s\n\n", messageID);
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 4th fwrite()");
			virge_syslog("FAILURE: Error occured during 4th fwrite() in virge_sendmail()");
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	snprintf(sendMsg, sizeof(sendMsg)-1, "%s\n", sendMessage);
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 5th fwrite()");
			virge_syslog("FAILURE: Error occured during 5th fwrite() in virge_sendmail()");
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	snprintf(sendMsg, sizeof(sendMsg)-1, ".\n");
	if (!fwrite(sendMsg, strlen(sendMsg), 1, sendmail))
	{
		if (ferror(sendmail) != 0)
		{
			virge_debug(2, "[ virge_sendmail() ] error during 6th fwrite()");
			virge_syslog("FAILURE: Error occured during 6th fwrite() in virge_sendmail()");
		}
	}
	else
	{
		virge_debug(4, "[ virge_sendmail() ] sent: [%s]", sendMsg);
	}

	/* Just to be safe? */
/*	sleep(1); */

	pclose(sendmail);
	virge_debug(1, "[ virge_sendmail() ] mail sent: FROM [%s] TO: [%s] SUBJECT: [%s]", VIRGE_FROM, email, sendSubject);

}
